﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.ComponentModel;

namespace CustomControls.DateTimePicker
{
    [ToolboxItem(false)]
    public class MonthCalendar : UserControl
    {
        readonly string[] Weekday = new string[] { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
        DateTime CurrentDate;
        DateTimePicker dateTimePicker;
        Label labPrevYear;
        Label labPrevMonth;
        Label labNextMonth;
        Label labNextYear;
        Label labCurrentDate;
        Label labSelectedDate;
        string prevYear = "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAKCAYAAACALL/6AAAABGdBTUEAALGPC/xhBQAAAF9JREFUKFNjQAe2UVHutuHhr4C0GVQIqxgYDBLF9pGRvrYREQ8co6KMoUJYxeDAPjraFGSSQ2SkB1QIqxgKoK8mu4iI1yAPQ4WwiqEAh6goA7vw8EuOkZH2UCEksUh7AHi0U/vJvVWtAAAAAElFTkSuQmCC",
            prevMonth = "iVBORw0KGgoAAAANSUhEUgAAAAYAAAALCAYAAABcUvyWAAAABGdBTUEAALGPC/xhBQAAAF1JREFUKFNjQAe2UVFmdhERJVAuBIAEbcPDXzmFhytDhRCCQNodKoRD0DkyUhxDEARsIyLu2EdGZkG5CGAfEWEL0uEQGekBFUIA++hoU/IlgZ57jeEQEAC7MiIiAwAzQymHZBUrOQAAAABJRU5ErkJggg==",
            nextMonth = "iVBORw0KGgoAAAANSUhEUgAAAAYAAAAKCAYAAACXDi8zAAAABGdBTUEAALGPC/xhBQAAAEdJREFUGFdjsIuI2ADE6xwaGlgYkAFIgAaSoatWMduGhy+3jYhYBhWCAN+0NC6gxBG78PAZUCGKBUEAKLge6JK5UC4UMDAAALD7LxOFqPajAAAAAElFTkSuQmCC",
            nextYear = "iVBORw0KGgoAAAANSUhEUgAAAAwAAAALCAYAAABLcGxfAAAABGdBTUEAALGPC/xhBQAAAI5JREFUKFNjsI2MDLYPD89MS0tjZYACbGIowDYiYpZdePhBt9hYbqgQVjEUAFYQEXHINy2NCyqEVQwFAJ0xG2QqsgJsYigArCAi4jSGJjQxOAAJ2oaH/3OOjBSHCmEVAwOQBMh6kIlQIaxiYECS4tDCQk50CWxicAD0UDe6BDYxOABKToYy4QCbGAMDAwMAcAFVx/DrCbkAAAAASUVORK5CYII=";

        private MonthCalendar()
        {
            labPrevYear = new Label() { Image = ConvertBase64ToImage(prevYear), Location = new Point(5, 5), Size = new Size(13, 13) };
            labPrevMonth = new Label() { Image = ConvertBase64ToImage(prevMonth), Location = new Point(23, 5), Size = new Size(13, 13) };
            labNextMonth = new Label() { Image = ConvertBase64ToImage(nextMonth), Location = new Point(144, 5), Size = new Size(13, 13) };
            labNextYear = new Label() { Image = ConvertBase64ToImage(nextYear), Location = new Point(162, 5), Size = new Size(13, 13) };

            labCurrentDate = new Label() { Location = new Point(41, 5), Size = new Size(98, 13), TextAlign = ContentAlignment.MiddleCenter };
            labSelectedDate = new Label() { Location = new Point(0, 162), Size = new Size(180, 18), TextAlign = ContentAlignment.MiddleCenter };

            this.Controls.Add(labPrevYear);
            this.Controls.Add(labPrevMonth);
            this.Controls.Add(labNextMonth);
            this.Controls.Add(labNextYear);

            foreach (Control item in this.Controls)
            {
                if (item is Label)
                {
                    item.Click += (sender, e) =>
                    {
                        if (sender == labPrevYear) { CurrentDate = CurrentDate.AddYears(-1); }
                        else if (sender == labPrevMonth) { CurrentDate = CurrentDate.AddMonths(-1); }
                        else if (sender == labNextMonth) { CurrentDate = CurrentDate.AddMonths(+1); }
                        else if (sender == labNextYear) { CurrentDate = CurrentDate.AddYears(+1); }

                        ShowDate(CurrentDate.Year, CurrentDate.Month);
                    };
                    item.MouseEnter += (sender, e) =>
                    {
                        ((Label)sender).BackColor = Color.FromArgb(247, 248, 249);
                    };
                    item.MouseLeave += (sender, e) =>
                    {
                        ((Label)sender).BackColor = Color.Transparent;
                    };
                }
            }
            this.Controls.Add(labCurrentDate);
            this.Controls.Add(labSelectedDate);

            this.Size = new Size(180, 180);
        }
        public MonthCalendar(DateTimePicker dateTimePicker) : this()
        {
            this.dateTimePicker = dateTimePicker;
            this.Font = dateTimePicker.Font;
            this.CurrentDate = dateTimePicker.Value;
            this.BackColor = dateTimePicker.MonthCalendarBackColor;
            DateInit();
            ShowDate(CurrentDate.Year, CurrentDate.Month);
            labSelectedDate.Text = Weekday[(int)DateTime.Today.DayOfWeek] + " , " + DateTime.Today.Year + "/" + DateTime.Today.Month + "/" + DateTime.Today.Day;
        }
        Image ConvertBase64ToImage(string base64String)
        {
            byte[] imageBytes = Convert.FromBase64String(base64String);
            using (MemoryStream ms = new MemoryStream(imageBytes, 0, imageBytes.Length))
            {
                ms.Write(imageBytes, 0, imageBytes.Length);
                return Image.FromStream(ms, true);
            }
        }


        private void DateInit()
        {
            #region 星期
            //周标题数组
            Label[] WeekScheme = new Label[7];
            for (int i = 0; i <= 6; i++)
            {
                WeekScheme[i] = new Label()
                {
                    Text = Weekday[i].Substring(1),
                    Width = 20,
                    Height = 16,
                    Font = this.Font,
                    Location = new Point(3 + i * 25, 25),
                    ForeColor = Color.FromArgb(85, 85, 85)
                };
            }
            WeekScheme[0].ForeColor = WeekScheme[6].ForeColor = Color.FromArgb(255, 128, 128);
            #endregion
            this.Controls.AddRange(WeekScheme);
            int DateNum = 0;
            Label[] DateDay = new Label[42];
            for (int i = 0; i < 6; i++)
            {
                int y = 47 + i * 18;
                for (int j = 0; j < 7; j++)
                {
                    DateDay[DateNum] = new Label
                    {
                        Name = "AutoLabDateDay" + DateNum.ToString(),
                        Width = 23,
                        Height = 14,
                        Font = this.Font,
                        Location = new Point(1 + j * 25, y),
                        TextAlign = ContentAlignment.MiddleRight
                    };
                    DateNum++;
                }
            }
            this.Controls.AddRange(DateDay);
        }
        private void ShowDate(int year, int month)
        {
            labCurrentDate.Text = year + "年" + month + "月";
            //                             1   2   3   4   5   6   7   8   9   10  11  12
            int[] DataDayLen = new int[] { 31, 30, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
            DataDayLen[1] = ((year % 4 == 0) && (year % 100 != 0) || (year % 400 == 0)) ? 29 : 28;
            DateTime MyDate = DateTime.Parse(year + "-" + month + "-" + 1);
            DateTime Today = DateTime.Today;
            int Day = (int)MyDate.DayOfWeek;

            for (int L = 0; L < 42; L++)
            {
                Label labDateDay = (Label)this.Controls["AutoLabDateDay" + L];
                labDateDay.Text = "";
                labDateDay.BackColor = Color.Transparent;
                //前一个月
                if (Day != 0 && L < Day)
                {
                    labDateDay.Text = (DataDayLen[MyDate.AddMonths(-1).Month - 1] - Day + L + 1).ToString();
                    labDateDay.ForeColor = Color.DarkGray;
                    labDateDay.Tag = DateTime.Parse(MyDate.AddMonths(-1).Year + "-" + MyDate.AddMonths(-1).Month + "-" + (DataDayLen[MyDate.AddMonths(-1).Month - 1] - Day + L + 1).ToString());
                }
                //当前月
                else if (L >= Day && L < DataDayLen[MyDate.Month - 1] + Day)
                {
                    labDateDay.Text = (L - Day + 1).ToString();
                    labDateDay.ForeColor = Color.FromArgb(85, 85, 85);
                    int day = Convert.ToInt32(labDateDay.Text);
                    labDateDay.Tag = DateTime.Parse(MyDate.Year + "-" + MyDate.Month + "-" + labDateDay.Text);
                }
                //下一个月
                else if (L >= DataDayLen[MyDate.Month - 1] + Day)
                {
                    labDateDay.Text = (L - DataDayLen[MyDate.Month - 1] - Day + 1).ToString();
                    labDateDay.ForeColor = Color.DarkGray;
                    labDateDay.Tag = DateTime.Parse(MyDate.AddMonths(+1).Year + "-" + MyDate.AddMonths(+1).Month + "-" + (L - DataDayLen[MyDate.Month - 1] - Day + 1).ToString());
                }

                labDateDay.MouseLeave += (sender, e) =>
                {
                    Label CuLabel = (Label)sender;
                    CuLabel.BackColor = Color.Transparent;
                };
                labDateDay.MouseClick += (sender, e) =>
                {
                    Label label = (Label)sender;
                    if (dateTimePicker != null)
                    {
                        DateTime time = Convert.ToDateTime(label.Tag);
                        dateTimePicker.Year = time.Year;
                        dateTimePicker.Month = time.Month;
                        dateTimePicker.Day = time.Day;
                        dateTimePicker.DropDownClose();
                    }
                };
                labDateDay.MouseEnter += (sender, e) =>
                {
                    Label label = (Label)sender;
                    label.BackColor = Color.FromArgb(247, 248, 249);
                    if (label.Tag != null)
                    {
                        DateTime time = Convert.ToDateTime(label.Tag);
                        labSelectedDate.Text = Weekday[(int)time.DayOfWeek] + " , " + time.Year + "-" + time.Month + "-" + time.Day;
                    }
                    else
                    {
                        labSelectedDate.Text = Weekday[(int)DateTime.Today.DayOfWeek] + " , " + DateTime.Today.Year + "-" + DateTime.Today.Month + "-" + DateTime.Today.Day;
                    }
                };
            }
        }
    }
}