﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using CustomControls;

namespace xwDateTimePicker.DateTimePicker
{
    [ToolboxItem(true)]
    public partial class xwDateTimePicker : UserControl
    {
        private int _Radius = 1;

        public int Radius
        {
            get { return _Radius; }
            set
            {
                if (value < 1) { throw new ArgumentOutOfRangeException(); }
                _Radius = value;
                this.Invalidate();
            }
        }
        private DateTime _value = DateTime.Now;

        public DateTime Value
        {
            get
            {
                _value = new DateTime(txtYear.Text.ToInt(2020), txtMonth.Text.ToInt(1), txtDay.Text.ToInt(1),
                    txtHour.Text.ToInt(1), txtMinite.Text.ToInt(1), txtSecond.Text.ToInt(1));
                Year = _value.Year;
                Month = _value.Month;
                Day = _value.Day;
                txtHour.Text = _value.Hour.ToString("00");
                txtMinite.Text = _value.Minute.ToString("00");
                txtSecond.Text = _value.Second.ToString("00");
                return _value;
            }
            set
            {
                Year = value.Year;
                Month = value.Month;
                Day = value.Day;
                txtHour.Text = value.Hour.ToString("00");
                txtMinite.Text = value.Minute.ToString("00");
                txtSecond.Text = value.Second.ToString("00");
                _value = value;
            }
        }
        enum SelectType
        {
            Year = 0,
            Month = 1,
            Day = 2,
            Hour = 3,
            Minute = 4,
            Second = 5
        }
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), EditorBrowsable(EditorBrowsableState.Never), Browsable(false)]
        public int Year
        {
            set { txtYear.Text = value + ""; }
        }
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), EditorBrowsable(EditorBrowsableState.Never), Browsable(false)]
        public int Month
        {
            set
            {
                if (value < 1 || value > 12) { throw new ArgumentOutOfRangeException("月份范围是1至12."); }
                txtMonth.Text = value.ToString("d2");
            }
        }
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), EditorBrowsable(EditorBrowsableState.Never), Browsable(false)]
        public int Day
        {
            set
            {
                if (value < 1 || value > 31) { throw new ArgumentOutOfRangeException("日期范围是1至31."); }
                txtDay.Text = value.ToString("d2");
            }
        }

        public xwDateTimePicker()
        {
            InitializeComponent();
            this.SetStyle(
           ControlStyles.UserPaint |  //控件自行绘制，而不使用操作系统的绘制
           ControlStyles.AllPaintingInWmPaint | //忽略擦出的消息，减少闪烁。
           ControlStyles.OptimizedDoubleBuffer |//在缓冲区上绘制，不直接绘制到屏幕上，减少闪烁。
           ControlStyles.ResizeRedraw | //控件大小发生变化时，重绘。                  
           ControlStyles.SupportsTransparentBackColor, true);//支持透明背景颜色


            txtYear.GotFocus += txtYear_GotFocus;
            txtMonth.GotFocus += txtYear_GotFocus;
            txtDay.GotFocus += txtYear_GotFocus;
            txtHour.GotFocus += txtYear_GotFocus;
            txtMinite.GotFocus += txtYear_GotFocus;
            txtSecond.GotFocus += txtYear_GotFocus;

            txtYear.Click += txtYear_GotFocus;
            txtMonth.Click += txtYear_GotFocus;
            txtDay.Click += txtYear_GotFocus;
            txtHour.Click += txtYear_GotFocus;
            txtMinite.Click += txtYear_GotFocus;
            txtSecond.Click += txtYear_GotFocus;

            txtYear.KeyDown += txtYear_KeyDown;
            txtMonth.KeyDown += txtYear_KeyDown;
            txtDay.KeyDown += txtYear_KeyDown;
            txtHour.KeyDown += txtYear_KeyDown;
            txtMinite.KeyDown += txtYear_KeyDown;
            txtSecond.KeyDown += txtYear_KeyDown;
            Value = DateTime.Now;
            RefershBack();
            this.BackColorChanged += delegate { RefershBack(); };
        }

        /// <summary>选择面板是否已经打开</summary>
        bool _dropDownShow = false;
        xwMonthCalendar _skinMonthCalendar;
        ToolStripDropDown _dropDown;
        private void txtYear_MouseDown(object sender, MouseEventArgs e)
        {
            if (!_dropDownShow)
            {
                _dropDownShow = true;
                _skinMonthCalendar = new xwMonthCalendar(this);

                _dropDown = new ToolStripDropDown() { Padding = Padding.Empty, Margin = Padding.Empty };
                _dropDown.Items.Add(new ToolStripControlHost(_skinMonthCalendar) { AutoSize = false, Padding = Padding.Empty, Margin = Padding.Empty });
                _dropDown.Show(this, new Point(0, this.Height));
                _dropDown.Closed += (s1, e1) => { _dropDownShow = false; };
            }
            else
            {
                _dropDownShow = false;
                DropDownClose();
            }
        }


        public void DropDownClose()
        {
            if (_dropDown != null) { if (!_dropDown.IsDisposed) { _dropDown.Close(); } }
            _skinMonthCalendar = null;
        }

        private void RefershBack()
        {
            if (this.BackColor == Color.Transparent)
            {
                txtHour.BackColor = txtMinite.BackColor = txtSecond.BackColor
                    = txtMonth.BackColor = txtDay.BackColor = txtYear.BackColor = Color.White;
            }
            else
            {
                txtHour.BackColor = txtMinite.BackColor = txtSecond.BackColor
                                  = txtMonth.BackColor = txtDay.BackColor = txtYear.BackColor = this.BackColor;
            }
        }

        void txtYear_KeyDown(object sender, KeyEventArgs e)
        {
            e.SuppressKeyPress = false;
            if (!(sender is TextBox _TextBox))
            {
                return;
            }
            string _txt = _TextBox.Text;
            SelectType _selected = SelectType.Year;
            switch (_TextBox.Name)
            {
                case "txtYear":
                    _selected = SelectType.Year;
                    break;
                case "txtMonth":
                    _selected = SelectType.Month;
                    break;
                case "txtDay":
                    _selected = SelectType.Day;
                    break;
                case "txtHour":
                    _selected = SelectType.Hour;
                    break;
                case "txtMinite":
                    _selected = SelectType.Minute;
                    break;
                case "txtSecond":
                    _selected = SelectType.Second;
                    break;
                default:
                    break;
            }

            if (e.KeyCode >= Keys.D0 && e.KeyCode <= Keys.D9)
            {

            }
            else if (e.KeyCode >= Keys.NumPad0 && e.KeyCode <= Keys.NumPad9)
            {
            }
            else if ((Keys)e.KeyValue == Keys.Delete)
            {
                if (_TextBox.SelectionLength > 0)
                {
                    _txt = _TextBox.Text.Substring(0, _TextBox.SelectionStart) + _TextBox.Text.Substring(_TextBox.SelectionStart + _TextBox.SelectionLength);
                }
                else if (_TextBox.SelectionStart >= 0)
                {
                    int _index = _TextBox.Text.Length - _TextBox.SelectionStart - 1;
                    _index = _index < 0 ? 0 : _index;
                    _txt = _txt.Substring(0, _TextBox.SelectionStart) + _txt.Substring(_TextBox.SelectionStart, _index);
                }
                _TextBox.Text = _txt;
                _TextBox.SelectionStart = _txt.Length;
                return;
            }
            else if ((Keys)e.KeyValue == Keys.Back)
            {
                if (_TextBox.SelectionLength > 0)
                {
                    _txt = _TextBox.Text.Substring(0, _TextBox.SelectionStart) + _TextBox.Text.Substring(_TextBox.SelectionStart + _TextBox.SelectionLength);
                }
                else if (_TextBox.SelectionStart >= 0)
                {
                    int _index = _TextBox.SelectionStart - 1;
                    _index = _index < 0 ? 0 : _index;
                    _txt = _txt.Substring(0, _index) + _txt.Substring(_TextBox.SelectionStart);
                }
                _TextBox.Text = _txt;
                _TextBox.SelectionStart = _txt.Length;
                return;
            }
            else
            {
                return;
            }
            if (_TextBox.SelectionLength > 0)
            {
                _txt = _txt.Substring(0, _TextBox.SelectionStart) + KeyName(e.KeyCode) + _txt.Substring(_TextBox.SelectionStart + _TextBox.SelectedText.Length);
            }
            else
            {
                if (_TextBox.SelectionStart >= 0)
                {
                    _txt = _txt.Substring(0, _TextBox.SelectionStart) + KeyName(e.KeyCode) + _txt.Substring(_TextBox.SelectionStart);
                }
                else
                {
                    _txt = _txt.Substring(0, _TextBox.SelectionStart) + KeyName(e.KeyCode);
                }

            }
            switch (_selected)
            {
                case SelectType.Year:
                    if (_txt.ToInt(0) > 0 && _txt.ToInt(0) < 9999)
                    { }
                    else
                    {
                        return;
                    }
                    break;
                case SelectType.Month:
                    if (_txt.ToInt(0) >= 0 && _txt.ToInt(0) <= 12)
                    { }
                    else
                    {
                        return;
                    }
                    break;
                case SelectType.Day:
                    DateTime dt = new DateTime(txtYear.Text.ToInt(1), txtMonth.Text.ToInt(1), 1, 0, 0, 0);
                    dt = dt.AddMonths(1).AddSeconds(-10);
                    int _Day = dt.Day;
                    if (_txt.ToInt(0) >= 0 && _txt.ToInt(0) <= _Day)
                    { }
                    else
                    {
                        return;
                    }
                    break;
                case SelectType.Hour:
                    if (_txt.ToInt(0) >= 0 && _txt.ToInt(0) < 24)
                    { }
                    else
                    {
                        return;
                    }
                    break;
                case SelectType.Minute:
                case SelectType.Second:
                    if (_txt.ToInt(0) >= 0 && _txt.ToInt(0) < 59)
                    { }
                    else
                    {
                        return;
                    }
                    break;
                default:
                    break;
            }
            _TextBox.Text = _txt;
            _TextBox.SelectionStart = _txt.Length;
        }

        char KeyName(Keys keycode)
        {
            if (keycode >= Keys.NumPad0 && keycode <= Keys.NumPad9)
            {
                char cc;
                switch (keycode)
                {
                    case Keys.NumPad0:
                    case Keys.D0:
                        cc = '0';
                        break;
                    case Keys.NumPad1:
                    case Keys.D1:
                        cc = '1';
                        break;
                    case Keys.NumPad2:
                    case Keys.D2:
                        cc = '2';
                        break;
                    case Keys.NumPad3:
                    case Keys.D3:
                        cc = '3';
                        break;
                    case Keys.NumPad4:
                    case Keys.D4:
                        cc = '4';
                        break;
                    case Keys.NumPad5:
                    case Keys.D5:
                        cc = '5';
                        break;
                    case Keys.NumPad6:
                    case Keys.D6:
                        cc = '6';
                        break;
                    case Keys.NumPad7:
                    case Keys.D7:
                        cc = '7';
                        break;
                    case Keys.NumPad8:
                    case Keys.D8:
                        cc = '8';
                        break;
                    case Keys.NumPad9:
                    case Keys.D9:
                        cc = '9';
                        break;
                    default: cc = ' '; break;
                }
                return cc;
            }
            else
            {
                return (char)keycode;
            }
        }

        void txtYear_GotFocus(object sender, EventArgs e)
        {
            if (sender is TextBox _txt)
            {
                _txt.SelectAll();
            }
        }

        protected override void OnForeColorChanged(EventArgs e)
        {
            base.OnForeColorChanged(e);
            txtSecond.ForeColor = txtMinite.ForeColor = txtHour.ForeColor = txtDay.ForeColor
                = txtMonth.ForeColor = txtYear.ForeColor = this.ForeColor;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            base.OnPaintBackground(e);
            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
            e.Graphics.CompositingQuality = CompositingQuality.HighQuality;
            e.Graphics.InterpolationMode = InterpolationMode.HighQualityBilinear;

            #region 根据内容和字体计算控件大小和位置

            this.Height = txtYear.ClientSize.Height * 2;
            int _locationY = (this.Height - txtYear.Height) / 2;
            _locationY = _locationY > 0 ? _locationY : 0;

            txtSecond.Font = txtMinite.Font = txtHour.Font = txtDay.Font = txtMonth.Font = txtYear.Font = this.Font;

            int widthHangxian = (int)e.Graphics.MeasureString("-", Font).Width;
            int widthMaohao = (int)e.Graphics.MeasureString(":", Font).Width;

            txtYear.Size = txtYear.PreferredSize;
            txtMonth.Size = txtMonth.PreferredSize;
            txtDay.Size = txtDay.PreferredSize;
            txtHour.Size = txtHour.PreferredSize;
            txtMinite.Size = txtMinite.PreferredSize;
            txtSecond.Size = txtSecond.PreferredSize;

            txtYear.Location = new Point(10, _locationY);
            txtMonth.Location = new Point(txtYear.Right + widthHangxian, _locationY);
            txtDay.Location = new Point(txtMonth.Right + widthHangxian, _locationY);
            txtHour.Location = new Point(txtDay.Right + 10, _locationY);
            txtMinite.Location = new Point(txtHour.Right + widthMaohao, _locationY);
            txtSecond.Location = new Point(txtMinite.Right + widthMaohao, _locationY);

            this.Width = txtSecond.Right + 10;
            #endregion
            Rectangle rect = new Rectangle(0, 0, this.Width, this.Height);
            var path = GetRoundedRectPath(rect, _Radius);

            this.Region = new Region(path);

            try
            {
                Graphics _Graphics = e.Graphics;
                {
                    using (SolidBrush b = new SolidBrush(BackColor))
                    {
                        _Graphics.FillPath(b, path);
                    }

                    SolidBrush _brush = new SolidBrush(txtYear.ForeColor);

                    int _widthLeft = 0;

                    //年和月之间的-
                    _Graphics.DrawString("-", Font, _brush, new PointF(_widthLeft + txtYear.Bounds.Right, _locationY));
                    _Graphics.DrawString("-", Font, _brush, new PointF(_widthLeft + txtMonth.Bounds.Right, _locationY));

                    //小时和分钟之间的:
                    _Graphics.DrawString(":", Font, _brush, new PointF(_widthLeft + txtHour.Bounds.Right, _locationY));
                    _Graphics.DrawString(":", Font, _brush, new PointF(_widthLeft + txtMinite.Bounds.Right, _locationY));
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
        GraphicsPath GetRoundedRectPath(Rectangle rect, int radius)
        {
            if (radius <= 0) { radius = 1; }

            int diameter = radius;
            Rectangle arcRect = new Rectangle(rect.Location, new Size(diameter, diameter));
            GraphicsPath path = new GraphicsPath();
            path.AddArc(arcRect, 180, 90);
            arcRect.X = rect.Right - diameter;
            path.AddArc(arcRect, 270, 90);
            arcRect.Y = rect.Bottom - diameter;
            path.AddArc(arcRect, 0, 90);
            arcRect.X = rect.Left;
            path.AddArc(arcRect, 90, 90);
            path.CloseFigure();
            return path;
        }
    }
}
