﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace xwDateTimePicker.DateTimePicker
{
    [ToolboxItem(false)]
    public partial class xwMonthCalendar : UserControl
    {
        #region 声明 
        readonly string[] Weekday = new string[] { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
        readonly DateTime[] TempDateDay = new DateTime[42];
        readonly xwDateTimePicker xwDateTimePicker;
        DateTime CurrentDate;
        #endregion
        readonly Font _font = new Font("微软雅黑", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Pixel);
        public xwMonthCalendar(xwDateTimePicker AlPicker)
        {
            InitializeComponent();
            this.BackColor = Color.White;
            xwDateTimePicker = AlPicker;
            this.CurrentDate = xwDateTimePicker.Value;
            _font = xwDateTimePicker.Font;
            DateInit();//初始设置
            ShowDate(CurrentDate.Year, CurrentDate.Month);
            labSelectedDate.Text = Weekday[(int)DateTime.Today.DayOfWeek] + " , " + DateTime.Today.Year + "/" + DateTime.Today.Month + "/" + DateTime.Today.Day;
        }
        private void DateInit()
        {
            #region 星期
            //周标题数组
            Label[] WeekScheme = new Label[7];
            string[] week = new string[] { "日", "一", "二", "三", "四", "五", "六" };
            for (int i = 0; i <= 6; i++)
            {
                WeekScheme[i] = new Label()
                {
                    Text = week[i],
                    Width = 20,
                    Height = 16,
                    Font = _font,
                    Location = new Point(3 + i * 25, 25),
                    ForeColor = Color.FromArgb(85, 85, 85)
                };
            }
            WeekScheme[0].ForeColor = WeekScheme[6].ForeColor = Color.FromArgb(255, 128, 128);
            #endregion
            this.Controls.AddRange(WeekScheme);
            int DateNum = 0;
            Label[] DateDay = new Label[42];
            for (int i = 0; i < 6; i++)
            {
                int y = 47 + i * 18;
                for (int j = 0; j < 7; j++)
                {
                    DateDay[DateNum] = new Label
                    {
                        Name = "AutoLabDateDay" + DateNum.ToString(),
                        Width = 23,
                        Height = 14,
                        Font = _font,
                        Location = new Point(1 + j * 25, y),
                        TextAlign = ContentAlignment.MiddleRight
                    };
                    DateNum++;
                }
            }
            this.Controls.AddRange(DateDay);
        }

        private void ShowDate(int year, int month)
        {
            labCurrentDate.Text = year + "年" + month + "月";
            //                             1   2   3   4   5   6   7   8   9   10  11  12
            int[] DataDayLen = new int[] { 31, 30, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
            DataDayLen[1] = ((year % 4 == 0) && (year % 100 != 0) || (year % 400 == 0)) ? 29 : 28;
            DateTime MyDate = DateTime.Parse(year + "-" + month + "-" + 1);
            DateTime Today = DateTime.Today;
            int Day = (int)MyDate.DayOfWeek;

            for (int L = 0; L < 42; L++)
            {
                Label labDateDay = (Label)this.Controls["AutoLabDateDay" + L];
                labDateDay.Text = "";
                labDateDay.BackColor = Color.Transparent;
                //前一个月
                if (Day != 0 && L < Day)
                {
                    labDateDay.Text = (DataDayLen[MyDate.AddMonths(-1).Month - 1] - Day + L + 1).ToString();
                    labDateDay.ForeColor = Color.DarkGray;
                    TempDateDay[L] = DateTime.Parse(MyDate.AddMonths(-1).Year + "-" + MyDate.AddMonths(-1).Month + "-" + (DataDayLen[MyDate.AddMonths(-1).Month - 1] - Day + L + 1).ToString());

                    labDateDay.Tag = TempDateDay[L];
                }
                //当前月
                else if (L >= Day && L < DataDayLen[MyDate.Month - 1] + Day)
                {
                    labDateDay.Text = (L - Day + 1).ToString();
                    labDateDay.ForeColor = Color.FromArgb(85, 85, 85);
                    int day = Convert.ToInt32(labDateDay.Text);
                    labDateDay.Tag = DateTime.Parse(MyDate.Year + "-" + MyDate.Month + "-" + labDateDay.Text);
                }
                //下一个月
                else if (L >= DataDayLen[MyDate.Month - 1] + Day)
                {
                    labDateDay.Text = (L - DataDayLen[MyDate.Month - 1] - Day + 1).ToString();
                    TempDateDay[L] = DateTime.Parse(MyDate.AddMonths(+1).Year + "-" + MyDate.AddMonths(+1).Month + "-" + (L - DataDayLen[MyDate.Month - 1] - Day + 1).ToString());
                    labDateDay.ForeColor = Color.DarkGray;
                    labDateDay.Tag = TempDateDay[L];
                }

                labDateDay.MouseLeave += (sender, e) =>
                {
                    Label CuLabel = (Label)sender;
                    CuLabel.BackColor = Color.Transparent;
                };
                labDateDay.MouseClick += (sender, e) =>
                {
                    Label label = (Label)sender;
                    if (xwDateTimePicker != null)
                    {
                        DateTime time = Convert.ToDateTime(label.Tag);
                        xwDateTimePicker.Year = time.Year;
                        xwDateTimePicker.Month = time.Month;
                        xwDateTimePicker.Day = time.Day;
                        xwDateTimePicker.DropDownClose();
                    }
                };
                labDateDay.MouseEnter += (sender, e) =>
                {
                    Label label = (Label)sender;
                    label.BackColor = Color.FromArgb(247, 248, 249);
                    if (label.Tag != null)
                    {
                        DateTime time = Convert.ToDateTime(label.Tag);
                        labSelectedDate.Text = Weekday[(int)time.DayOfWeek] + " , " + time.Year + "-" + time.Month + "-" + time.Day;
                    }
                    else
                    {
                        labSelectedDate.Text = Weekday[(int)DateTime.Today.DayOfWeek] + " , " + DateTime.Today.Year + "-" + DateTime.Today.Month + "-" + DateTime.Today.Day;
                    }
                };
            }
        }

        private void LabPrevYear_Click(object sender, EventArgs e)
        {
            if (sender == labPrevYear) { CurrentDate = CurrentDate.AddYears(-1); }
            else if (sender == labPrevMonth) { CurrentDate = CurrentDate.AddMonths(-1); }
            else if (sender == labNextMonth) { CurrentDate = CurrentDate.AddMonths(+1); }
            else if (sender == labNextYear) { CurrentDate = CurrentDate.AddYears(+1); }

            ShowDate(CurrentDate.Year, CurrentDate.Month);
        }

        private void LabPrevYear_MouseEnter(object sender, EventArgs e)
        {
            ((Label)sender).BackColor = Color.FromArgb(247, 248, 249);
        }

        private void LabPrevYear_MouseLeave(object sender, EventArgs e)
        {
            ((Label)sender).BackColor = Color.Transparent;
        }
    }
}
